<DefaultConfig>
    CONFIG_DIR = /usr/share/YaTeA/config
    LOCALE_DIR = /usr/share/YaTeA/locale
    RESULT_DIR = $PWD/RESULTS
</DefaultConfig>
<OPTIONS>
    MESSAGE_DISPLAY = EN
    TC-for-BioLG = 0
    TT-for-BioLG = 0
    TTG-style-term-candidates = multi
    XML-corpus-for-BioLG = 0
    annotate-only = 0
    debug = 0
    language = FR
    match-type = 
    monolexical-all = 0
    monolexical-included = 0
    printChunking = 0
    suffix = default
    termList = multi
    xmlout = 1
</OPTIONS>
